/*========================================================================
 * io.c  --  Disk I/O routines for Tsunami server.
 *
 * This contains disk I/O routines for the Tsunami file transfer server.
 *
 * Written by Mark Meiss (mmeiss@indiana.edu).
 * Copyright  2002 The Trustees of Indiana University.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) All redistributions of source code must retain the above
 *    copyright notice, the list of authors in the original source
 *    code, this list of conditions and the disclaimer listed in this
 *    license;
 *
 * 2) All redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the disclaimer
 *    listed in this license in the documentation and/or other
 *    materials provided with the distribution;
 *
 * 3) Any documentation included with all redistributions must include
 *    the following acknowledgement:
 *
 *      "This product includes software developed by Indiana
 *      University`s Advanced Network Management Lab. For further
 *      information, contact Steven Wallace at 812-855-0960."
 *
 *    Alternatively, this acknowledgment may appear in the software
 *    itself, and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4) The name "tsunami" shall not be used to endorse or promote
 *    products derived from this software without prior written
 *    permission from Indiana University.  For written permission,
 *    please contact Steven Wallace at 812-855-0960.
 *
 * 5) Products derived from this software may not be called "tsunami",
 *    nor may "tsunami" appear in their name, without prior written
 *    permission of Indiana University.
 *
 * Indiana University provides no reassurances that the source code
 * provided does not infringe the patent or any other intellectual
 * property rights of any other entity.  Indiana University disclaims
 * any liability to any recipient for claims brought by any other
 * entity based on infringement of intellectual property rights or
 * otherwise.
 *
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
 * NOWARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
 * SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT,
 * OR OTHER PROPRIETARY RIGHTS. INDIANA UNIVERSITY MAKES NO
 * WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN
 * HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE
 * ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE.
 *========================================================================*/

#include "server.h"


/*------------------------------------------------------------------------
 * int build_datagram(ttp_session_t *session, u_int32_t block_index,
 *                    u_int16_t block_type, u_char *datagram);
 *
 * Constructs to hold the given block of data, with the given type
 * stored in it.  The format of the datagram is:
 *
 *     32                    0
 *     +---------------------+
 *     |     block_number    |
 *     +----------+----------+
 *     |   type   |   data   |
 *     +----------+     :    |
 *     :     :          :    :
 *     +---------------------+
 *
 * The datagram is stored in the given buffer, which must be at least
 * six bytes longer than the block size for the transfer.  Returns 0 on
 * success and non-zero on failure.
 *------------------------------------------------------------------------*/
int build_datagram(ttp_session_t *session, u_int32_t block_index,
		   u_int16_t block_type, u_char *datagram)
{
    u_int32_t        block_size = session->parameter->block_size;
    static u_int32_t last_block = 0;
    u_int32_t       ringbuf_pointer;
    int              status;
    u_int32_t        write_size;



    ringbuf_pointer = ((block_index-1) % RINGBUF_BLOCKS) 
         * session->parameter->block_size;


    while (block_index > last_block){ /* read data from vsib, if not in memory */
         read_vsib_block(session->parameter->ringbuf + ringbuf_pointer, 
             session->parameter->block_size);
         last_block++;
         if(session->parameter->fileout){

             /* figure out how many bytes to write */
          write_size = (block_index == session->parameter->block_count) ? 
             (session->parameter->file_size % block_size) : block_size;
          if (write_size == 0)
             write_size = block_size;
	   /* write the block to disk */
	   /*	   status = fwrite((session->parameter->ringbuf + ringbuf_pointer,
	       1,write_size, 
               session->transfer.file); */
	   status = fwrite(session->parameter->ringbuf + ringbuf_pointer, 1, 
              write_size, session->transfer.file);


	   if (status < write_size) {
	     sprintf(g_error, "Could not write block %d of file", block_index);
	     return warn(g_error);
	     }
	   }
    }

    if (last_block - block_index > RINGBUF_BLOCKS) {
      /*printf("Block #%u expired", block_index);*//*already expired */
	printf(".");/*already expired */
        /* fill with zero */
        memset(datagram + 6, 0, session->parameter->block_size); 
	/* return warn(g_error); */  /* continue anyway */
        }
    else
        memcpy(datagram +6, session->parameter->ringbuf + ringbuf_pointer,
            session->parameter->block_size);


    /* build the datagram header */
    *((u_int32_t *) (datagram + 0)) = htonl(block_index);
    *((u_int16_t *) (datagram + 4)) = htons(block_type);

    /* return success */
    return 0;
}


/*========================================================================
 * $Log$
 */
